@@ -17,7 +17,7 @@
 uint32_t g_security_status;
 string g_security_message;
 
-void doSecPoll(time_t* last_secpoll)
+void doSecPoll(time_t* last_secpoll, Logr::log_t log)
 {
   if (::arg()["security-poll-suffix"].empty())
     return;
@@ -55,15 +55,18 @@ void doSecPoll(time_t* last_secpoll)
     state = sr.getValidationState();
   }
 
+  auto vlog = log->withValues("version", Logging::Loggable(pkgv), "query", Logging::Loggable(query));
   if (vStateIsBogus(state)) {
-    g_log << Logger::Error << "Failed to retrieve security status update for '" + pkgv + "' on '" << query << "', DNSSEC validation result was Bogus!" << endl;
+    SLOG(g_log << Logger::Error << "Failed to retrieve security status update for '" + pkgv + "' on '" << query << "', DNSSEC validation result was Bogus!" << endl,
+         vlog->info(Logr::Error, "Failed to retrieve security status update", "validationResult", Logging::Loggable(vStateToString(state))));
     if (g_security_status == 1) // If we were OK, go to unknown
       g_security_status = 0;
     return;
   }
 
   if (res == RCode::NXDomain && !isReleaseVersion(pkgv)) {
-    g_log << Logger::Warning << "Not validating response for security status update, this is a non-release version" << endl;
+    SLOG(g_log << Logger::Warning << "Not validating response for security status update, this is a non-release version" << endl,
+         vlog->info(Logr::Warning, "Not validating response for security status update, this is a non-release version"));
     return;
   }
 
@@ -75,20 +78,25 @@ void doSecPoll(time_t* last_secpoll)
   }
   catch (const PDNSException& pe) {
     g_security_status = security_status;
-    g_log << Logger::Warning << "Failed to retrieve security status update for '" << pkgv << "' on '" << query << "': " << pe.reason << endl;
+    SLOG(g_log << Logger::Warning << "Failed to retrieve security status update for '" << pkgv << "' on '" << query << "': " << pe.reason << endl,
+         vlog->error(Logr::Warning, pe.reason, "Failed to retrieve security status update"));
     return;
   }
 
   g_security_message = security_message;
 
+  auto rlog = vlog->withValues("message", Logging::Loggable(g_security_message), "status", Logging::Loggable(security_status));
   if (g_security_status != 1 && security_status == 1) {
-    g_log << Logger::Warning << "Polled security status of version " << pkgv << ", no known issues reported: " << g_security_message << endl;
+    SLOG(g_log << Logger::Warning << "Polled security status of version " << pkgv << ", no known issues reported: " << g_security_message << endl,
+         rlog->info(Logr::Notice, "Polled security status of version, no known issues reported"));
   }
   if (security_status == 2) {
-    g_log << Logger::Error << "PowerDNS Security Update Recommended: " << g_security_message << endl;
+    SLOG(g_log << Logger::Error << "PowerDNS Security Update Recommended: " << g_security_message << endl,
+         rlog->info(Logr::Error, "PowerDNS Security Update Recommended"));
   }
   if (security_status == 3) {
-    g_log << Logger::Error << "PowerDNS Security Update Mandatory: " << g_security_message << endl;
+    SLOG(g_log << Logger::Error << "PowerDNS Security Update Mandatory: " << g_security_message << endl,
+         rlog->info(Logr::Error, "PowerDNS Security Update Mandatory"));
   }
 
   g_security_status = security_status;
